/* popUp multiPlane ground attack. V0.06.05 2008/01/13                      */
/* coordConv.c coordinate conversion functions                              */
/* Copyright  2008 Author: Valerio Messina I-EFAA http://users.iol.it/efa  */
/* This file is part of PopUp.
   PopUp is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   PopUp is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.          */

#include "stdio.h"
#include "math.h"

#include "comType.h"
#include "pupData.h"
#include "coord.h"

#if (pupDbg>=PRINTF)
/* Remove: converte i radianti di latitudine in metri sulla superficie terrestre */
errOk rad2lat (double* radPtr, double* latPtr) {
   if (radPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s radPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (latPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s latPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if ((*radPtr<-M_PI/2.0)||(*radPtr>M_PI/2.0)) { // -90 +90
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad:%f out of range -PI/2:PI/2\n", __FUNCTION__, *radPtr);
      return (ERROR);
   }
   *latPtr = EarthRadius * *radPtr;
   return (OK);
}

/* Remove: converte i radianti di longitudine in metri sulla superficie terrestre */
errOk rad2lon (double* radPtr, double* lonPtr) {
   if (radPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s radPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (lonPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s lonPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if ((*radPtr<-M_PI)||(*radPtr>M_PI)) { // -180 +180
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad:%f out of range -PI:PI\n", __FUNCTION__, *radPtr);
      return (ERROR);
   }
   *lonPtr = EarthRadius * *radPtr;
   return (OK);
}
#endif

/* converte da cartesiane (radianti,radianti) a cartesiane (metri,metri) */
errOk rad2metr (coordTy* coordPtr) {
   if (coordPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s coordPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if ((coordPtr->latRad<-M_PI/2.0)||(coordPtr->latRad>M_PI/2.0)) { // -90 +90
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad:%f out of range -PI/2:PI/2\n", __FUNCTION__, coordPtr->latRad);
      return (ERROR);
   }
   if ((coordPtr->lonRad<-M_PI)||(coordPtr->lonRad>M_PI)) { // -180 +180
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad:%f out of range -PI:PI\n", __FUNCTION__, coordPtr->lonRad);
      return (ERROR);
   }
   coordPtr->lon = coordPtr->lonRad * EarthRadius;
   coordPtr->lat = coordPtr->latRad * EarthRadius;
   return (OK);
}

/* converte da cartesiane (metri,metri) a cartesiane (radianti,radianti) */
errOk metr2rad (coordTy* coordPtr) {
   if (coordPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s coordPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (pupDbg>=PRINTALL) printf ("%s lat:%16.6f M_PI/2.0*EarthRadius:%16.6f\n", __FUNCTION__, coordPtr->lat, M_PI/2.0*EarthRadius);
   if (pupDbg>=PRINTALL) printf ("%s lon:%16.6f M_PI*EarthRadius    :%16.6f\n", __FUNCTION__, coordPtr->lon, M_PI*EarthRadius);
   if ((coordPtr->lat<-M_PI/2.0*EarthRadius)||(coordPtr->lat>M_PI/2.0*EarthRadius)) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s lat:%f out of range -PI/2:PI/2\n", __FUNCTION__, coordPtr->lat);
      return (ERROR);
   }
   if ((coordPtr->lon<-M_PI*EarthRadius)||(coordPtr->lon>M_PI*EarthRadius)) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s lon:%f out of range -PI:PI\n", __FUNCTION__, coordPtr->lon);
      return (ERROR);
   }
   coordPtr->latRad = coordPtr->lat / EarthRadius;
   coordPtr->lonRad = coordPtr->lon / EarthRadius;
   return (OK);
}

/* converte da cartesiane (metri,metri) a polariGeo (metri,rad) */
/* le polariGeo hanno gli 0 verso l'alto, e aumentano in senso orario */
errOk cart2pol (coordTy* coordPtr) {
   coordTy coord;
   double bea;
   if (coordPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s coordPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   coord = *coordPtr;
   if ((coord.lat<-M_PI/2.0*EarthRadius)||(coord.lat>M_PI/2.0*EarthRadius)) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s lat:%f out of range -PI/2:PI/2\n", __FUNCTION__, coord.lat);
      return (ERROR);
   }
   if ((coord.lon<-M_PI*EarthRadius)||(coord.lon>M_PI*EarthRadius)) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s lon:%f out of range -PI:PI\n", __FUNCTION__, coord.lon);
      return (ERROR);
   }
   coord.ran = sqrt (coord.lon*coord.lon + coord.lat*coord.lat);
   if ((coord.lat==0)&&(coord.lon==0)) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s coord is 0,0\n", __FUNCTION__);
      coord.bea = 0.0;
      *coordPtr = coord;
      return (ERROR);
   }
   coord.bea = atan2 (coord.lat, coord.lon);
   bea = M_PI/2.0 - coord.bea; // 0 nord verso l'alto in geografia
   if (bea<0.0) bea += 2*M_PI; // 0 verso dx in matematica
   coord.bea = bea;
   *coordPtr = coord;
   return (OK);
}

/* converte da polariGeo (metri,rad) a cartesiane (metri,metri) */
/* le polariGeo hanno gli 0 verso l'alto, e aumentano in senso orario */
errOk pol2cart (coordTy* coordPtr) {
   coordTy coord;
   double bea;
   if (coordPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s coordPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   coord = *coordPtr;
   bea = M_PI/2.0 - coord.bea; // 0 nord verso l'alto in geografia
   if (bea<0.0) bea += 2*M_PI; // 0 verso dx in matematica
   coord.lat = coord.ran * sin (bea);
   coord.lon = coord.ran * cos (bea);
   if ((coord.lat<-M_PI/2.0*EarthRadius)||(coord.lat>M_PI/2.0*EarthRadius)) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s lat:%f out of range -PI/2:PI/2\n", __FUNCTION__, coord.lat);
      return (ERROR);
   }
   if ((coord.lon<-M_PI*EarthRadius)||(coord.lon>M_PI*EarthRadius)) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s lon:%f out of range -PI:PI\n", __FUNCTION__, coord.lon);
      return (ERROR);
   }
   *coordPtr = coord;
   return (OK);
}

/* ,  ',  ' " in metri lat/lon */
/* metri lat/lon in ,  ',  ' " */

#if (pupDbg>=PRINTF)
int try () {
   double rad;
   degTy deg;
   errOk ret;
   for (rad=0.0; rad<2*M_PI; rad+=1.0) {
      ret = rad2deg (&rad, &deg);
      if (ret!=OK) {
         if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned not OK\n", __FUNCTION__);
         return (ERROR);
      }
      printf ("rad:%1.0f ", rad);
      printf ("deg:%10.6f pri:%7.4f sec:%7.4f ", deg.deg, deg.pri, deg.sec);
      printf ("degInt:%3u priInt:%2u secInt:%2u\n", deg.degInt, deg.priInt, deg.secInt);
   }
   return (OK);
}
#endif
