/* popUp multiPlane ground attack. V0.06.05 2008/01/13                      */
/* graph.c draw the graph using the Cairo vectorial library                 */
/* Copyright  2008 Author: Valerio Messina I-EFAA http://users.iol.it/efa  */
/* This file is part of PopUp.
   PopUp is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   PopUp is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.          */

#include <math.h>

#include <gtk/gtk.h>

#include "comType.h"                         // inutile gi incluso da coord.h
#include "coord.h"                         // inutile gi incluso da pupData.h
#include "pupData.h"

/* memorizza se il tracciato di un aereo  attivo o no */
bit planesOn[Planes];

/* determina le massime distanze dei punti del grafico di un solo aereo */
errOk calcPlaneDist (pointTy planePoints[], double* distXaxesPtr, double* distYaxesPtr) {
   double absX, maxX = 0.0;
   double absY, maxY = 0.0;
   if (planePoints==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s planePoints point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (distXaxesPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s distXaxesPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (distYaxesPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s distYaxesPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   for (u08 point=0; point<Points; point++) {
      if (pupDbg>=PRINTVERBOSE) printf ("%s lat:%f lon:%f\n", __FUNCTION__, planePoints[point].rel.lat, planePoints[point].rel.lon);
      if (pupDbg>=PRINTVERBOSE) printf ("%s maxY:%f maxX:%f\n", __FUNCTION__, maxY, maxX);
      absY = fabs (planePoints[point].rel.lat);
      if (absY > maxY) {
         maxY = absY;
      }
      absX = fabs (planePoints[point].rel.lon);
      if (absX > maxX) {
         maxX = absX;
      }
      if (pupDbg>=PRINTVERBOSE) printf ("%s maxY:%f maxX:%f\n", __FUNCTION__, maxY, maxX);
   }

   *distXaxesPtr = maxX;
   *distYaxesPtr = maxY;
   return (OK);
} /* errOk calcPlaneDist (pointTy planePoints[], double* distXaxesPtr, double* distYaxesPtr) */

/* determina le massime distanze dei punti dei grafici di tutti gli aerei attivi */
errOk calcGraphDist (pointTy points[Planes][Points], double* widthPtr, double* heightPtr) {
   double maxX = 0.0;
   double maxY = 0.0;
   double width, height;
   errOk ret;
   if (points==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s points point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (widthPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s widthPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (heightPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s heightPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   for (u08 plane=0; plane<Planes; plane++) {
      if (!planesOn[plane]) continue; // salta gli aerei spenti
      ret = calcPlaneDist (points[plane], &width, &height);
      if (ret!=OK) {
         if (pupDbg>=PRINTERROR) printf ("ERROR %s calcPlaneDist returned:%d not OK\n", __FUNCTION__, ret);
         return (ERROR);
      }
      // ora confronto la massima larghezza e altezza di un aereo con gli altri
      if (height > maxY) {
         maxY = height;
      }
      if (width > maxX) {
         maxX = width;
      }
      if (pupDbg>=PRINTVERBOSE) printf ("%s plane:%u width:%f height:%f\n", __FUNCTION__, plane, width, height);
   }
   *widthPtr = maxX;
   *heightPtr = maxY;
   if (pupDbg>=PRINTVERBOSE) printf ("\n");
   return (OK);
} /* errOk calcGraphDist (pointTy points[Planes][Points], double* widthPtr, double* heightPtr) */

/* calcola il fattore di scala da usare nel disegno per farci stare tutti i path */
int calcGraphScale (double pathWidth, double pathHeight, gint winWidth, gint winHeight, double* scalePtr) {
   double scaleX, scaleY;
   if (winWidth<=0) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s winWidth:%u must be >0\n", __FUNCTION__, winWidth);
      return (ERROR);
   }
   if (winHeight<=0) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s winHeight:%u must be >0\n", __FUNCTION__, winHeight);
      return (ERROR);
   }
   if (scalePtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s scalePtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   scaleX = pathWidth  / winWidth;
   scaleY = pathHeight / winHeight;
   *scalePtr = scaleY;               // inizialmente supporne scaleX <= scaleY
   if (scaleX > scaleY) *scalePtr = scaleX; // caso opposto
   return (OK);
} /* int calcGraphScale (double pathWidth, double pathHeight, gint winWidth, gint winHeight, double* scalePtr) */

#if (pupDbg>=PRINTF)
int test () { // remove
   double pathWidth, pathHeight;
   double scale;
   gint winWidth = 200, winHeight = 200;
   errOk ret;

   ret = calcCoords (0);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s calcCoords returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTF) printf ("\n");
   ret = calcGraphDist (points, &pathWidth, &pathHeight);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s calcGraphDist returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   pathWidth  *= 2.0; // la dimensione del grafico  il doppio della max distance
   pathHeight *= 2.0; // la dimensione del grafico  il doppio della max distance
   if (pupDbg>=PRINTF) printf ("path width:%f height:%f\n", pathWidth, pathHeight);
   ret = calcGraphScale (pathWidth, pathHeight, winWidth, winHeight, &scale);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s calcGraphScale returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTF) printf ("win width:%d height:%d\n", winWidth, winHeight);
   if (pupDbg>=PRINTF) printf ("scale:%f\n", scale);
   return (OK);
}
#endif

/* disegna il sistema di coordinate polari */
errOk drawAxis (cairo_t* drawPtr, double ox, double oy, double width, double height) {
   double radius;
   if (drawPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s drawPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   cairo_set_line_width (drawPtr, 1.0); // 1 pixel
   cairo_set_source_rgb (drawPtr, 0.0, 0.0, 0.0); // nero
   cairo_rectangle (drawPtr, ox, oy, width, height);
   //cairo_clip (drawPtr);
   cairo_set_source_rgb (drawPtr, 1.0, 1.0, 1.0); // bianco
   cairo_fill_preserve (drawPtr);
   cairo_set_source_rgb (drawPtr, 0.0, 0.0, 0.0); // nero again. Why?
   cairo_stroke (drawPtr);

   cairo_move_to (drawPtr, ox      , oy+height/2.0);
   cairo_line_to (drawPtr, ox+width, oy+height/2.0); // x axis
   cairo_stroke (drawPtr);

   cairo_move_to (drawPtr, ox+width/2.0, oy);
   cairo_line_to (drawPtr, ox+width/2.0, oy+height); // y axis
   cairo_stroke (drawPtr);

   radius = MIN (width/2.0, height/2.0); // disegna i cerchi concentrici
   cairo_arc (drawPtr, ox+width/2.0, oy+height/2.0, radius, 0.0, 2*M_PI); // disegna un cerchio: arco 0-2PI, centrato e raggio radius
   cairo_stroke (drawPtr);
   radius /= 3.0;
   cairo_arc (drawPtr, ox+width/2.0, oy+height/2.0, radius, 0.0, 2*M_PI); // disegna un cerchio: arco 0-2PI, centrato e raggio radius
   cairo_stroke (drawPtr);
   radius *= 2.0;
   cairo_arc (drawPtr, ox+width/2.0, oy+height/2.0, radius, 0.0, 2*M_PI); // disegna un cerchio: arco 0-2PI, centrato e raggio radius
   cairo_stroke (drawPtr);

   return (OK);
} /* errOk drawAxis (cairo_t* drawPtr, double ox, double oy, double width, double height) */

/* disegna il simbolo di un punto */
errOk drawPoint (cairo_t* drawPtr, double x, double y, double scale, u08 plane, u08 point) {
   double radius;
   if (drawPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s drawPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (plane>=Planes) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s plane:%u must be <%u\n", __FUNCTION__, plane, Planes);
      return (ERROR);
   }
   if (point>=Points) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s point:%u must be <%u\n", __FUNCTION__, point, Points);
      return (ERROR);
   }

   radius = 10.0; // scale vary about from 380 (smallest) to 10 (bigger)
   switch (plane) {
   case 0:
      cairo_set_source_rgb (drawPtr, 1.0, 0.0, 0.0); // rosso
      break;
   case 1:
      cairo_set_source_rgb (drawPtr, 0.0, 1.0, 0.0); // verde
      break;
   case 2:
      cairo_set_source_rgb (drawPtr, 0.0, 0.0, 1.0); // blu
      break;
   case 3:
      cairo_set_source_rgb (drawPtr, 1.0, 0.0, 1.0); // viola
      break;
   default:
      if (pupDbg>=PRINTERROR) printf ("ERROR %s plane:%u should be <%u\n", __FUNCTION__, plane, Planes);
      return (ERROR);
   }
   switch (point) {
   case 0:
      
      //break;
   case 1:
      
      //break;
   case 2:
      
      //break;
   case 3:
      
      //break;
   case 4:
      
      //break;
   case 5:
      
      //break;
   case 6:
      cairo_arc (drawPtr, x, y, radius, 0.0, 2*M_PI); // disegna un cerchio: arco 0-2PI, centrato e raggio radius
      break;
   default:
      if (pupDbg>=PRINTERROR) printf ("ERROR %s point:%u should be <%u\n", __FUNCTION__, point, Points);
      return (ERROR);
   }
   cairo_stroke (drawPtr);

   return (OK);
} /* errOk drawPoint (cairo_t* drawPtr, double x, double y, double scale, u08 plane, u08 point) */

/* disegna il path di un aereo */
errOk drawPath (cairo_t* drawPtr, double scale, double xOff, double yOff, double winEdge, gint winHeight, u08 plane) {
   double x, y;
   if (drawPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s drawPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (scale<=0.0) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s scale:%f must be >0\n", __FUNCTION__, scale);
      return (ERROR);
   }
   if (plane>=Planes) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s plane:%u must be <%u\n", __FUNCTION__, plane, Planes);
      return (ERROR);
   }
   x = points[plane][0].rel.lon / scale + xOff + winEdge/2.0;
   y = points[plane][0].rel.lat / scale + yOff + winEdge/2.0;
   y = winHeight - y;
   cairo_move_to (drawPtr, x, y);
   for (u08 point=0; point<Points; point++) {
      x = points[plane][point].rel.lon / scale + xOff + winEdge/2.0;
      y = points[plane][point].rel.lat / scale + yOff + winEdge/2.0;
      y = winHeight - y;
      cairo_line_to (drawPtr, x, y);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("%s plane:%u\n", __FUNCTION__, plane);
   switch (plane) {
   case 0:
      cairo_set_source_rgb (drawPtr, 1.0, 0.0, 0.0); // rosso
      break;
   case 1:
      cairo_set_source_rgb (drawPtr, 0.0, 1.0, 0.0); // verde
      break;
   case 2:
      cairo_set_source_rgb (drawPtr, 0.0, 0.0, 1.0); // blu
      break;
   case 3:
      cairo_set_source_rgb (drawPtr, 1.0, 0.0, 1.0); // viola
      break;
   default:
      if (pupDbg>=PRINTERROR) printf ("ERROR %s plane:%u should be <%u\n", __FUNCTION__, plane, Planes);
      return (ERROR);
   }
   cairo_stroke (drawPtr);
   return (OK);
} /* errOk drawPath (cairo_t* drawPtr, double scale, double xOff, double yOff, double winEdge, gint winHeight, u08 plane) */

/* visualizza tutto il canvas, completo di path di tutti gli aerei attivi */
errOk exposeCanvas (GtkWidget* canvasPtr, GdkEventExpose* eventPtr) {
   cairo_t* drawPtr;
   double pathWidth, pathHeight;
   double scale;
   gint winWidth, winHeight;
   double winEdge;
   double xOff, yOff;
   double x, y;
   errOk ret;

   if (canvasPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s canvasPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (eventPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s eventPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("canvasPtr point to:%p\n", canvasPtr);
   if (pupDbg>=PRINTVERBOSE) printf ("eventPtr point to:%p\n", eventPtr);

   drawPtr = gdk_cairo_create (canvasPtr->window); // was eventPtr
   //x = canvasPtr->allocation.x;
   //y = canvasPtr->allocation.y;
   winWidth  = canvasPtr->allocation.width;
   winHeight = canvasPtr->allocation.height;

   for (u08 plane=0; plane<Planes; plane++) {
      if (!planesOn[plane]) continue; // salta gli aerei spenti
      ret = calcCoords (plane); /* calcola tutti i valori necessari al disegno */
      if (ret!=OK) {
         if (pupDbg>=PRINTERROR) printf ("ERROR %s calcCoords returned:%d not OK\n", __FUNCTION__, ret);
         return (ERROR);
      }
   }
   if (pupDbg>=PRINTVERBOSE) printf ("\n");

   // calcola la distanza del punto pi lontano
   ret = calcGraphDist (points, &pathWidth, &pathHeight);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s calcGraphDist returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   pathWidth  *= 2.0; // la dimensione del grafico  il doppio della max distance
   pathHeight *= 2.0; // la dimensione del grafico  il doppio della max distance
   if (pupDbg>=PRINTVERBOSE) printf ("%s path width:%f height:%f\n", __FUNCTION__, pathWidth, pathHeight);
   winEdge = MIN (winWidth, winHeight) - 5.0; // lato del quadrato, lascia 5 pixel
   // calcola la scala in cui disegnare il grafico
   ret = calcGraphScale (pathWidth, pathHeight, winEdge, winEdge, &scale);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s calcGraphScale returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("%s win width:%d height:%d winEdge:%f\n", __FUNCTION__, winWidth, winHeight, winEdge);
   if (pupDbg>=PRINTVERBOSE) printf ("%s scale:%f\n", __FUNCTION__, scale);
   if (scale<=0.0) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s scale:%f must be >0\n", __FUNCTION__, scale);
      return (ERROR);
   }

   // disegna gli assi delle coordinate
   xOff = (winWidth -winEdge)/2.0;
   yOff = (winHeight-winEdge)/2.0;
   ret = drawAxis (drawPtr, xOff, yOff, winEdge, winEdge);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s drawAxis returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }

   // disegna i simboli dei punti degli aerei
   for (u08 plane=0; plane<Planes; plane++) {
      if (!planesOn[plane]) continue; // salta gli aerei spenti
      for (u08 point=0; point<Points; point++) {
         x = points[plane][point].rel.lon / scale + xOff + winEdge/2.0;
         y = points[plane][point].rel.lat / scale + yOff + winEdge/2.0;
         y = winHeight - y; // i display hanno la coord y crescente vs il basso!?!
         ret = drawPoint (drawPtr, x, y, scale, plane, point);
         if (ret!=OK) {
            if (pupDbg>=PRINTERROR) printf ("ERROR %s drawPoint returned:%d not OK\n", __FUNCTION__, ret);
            return (ERROR);
         }
      }
   }

   // disegna le linee dei percorsi
   for (u08 plane=0; plane<Planes; plane++) {
      if (!planesOn[plane]) continue; // salta gli aerei spenti
      ret = drawPath (drawPtr, scale, xOff, yOff, winEdge, winHeight, plane);
      if (ret!=OK) {
         if (pupDbg>=PRINTERROR) printf ("ERROR %s drawPath returned:%d not OK\n", __FUNCTION__, ret);
         return (ERROR);
      }
   }

   cairo_destroy (drawPtr);
   return (OK);
} /* errOk exposeCanvas (GtkWidget* canvasPtr, GdkEventExpose* eventPtr) */

#if (pupDbg>=PRINTF)
int mainTest (int argc, char** argv) { // remove
   GtkWidget* windowPtr;
   GtkWidget* canvasPtr;

   gtk_init (&argc, &argv);

   windowPtr = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   canvasPtr = gtk_drawing_area_new ();
   gtk_container_add (GTK_CONTAINER (windowPtr), canvasPtr);

   g_signal_connect (windowPtr, "destroy", G_CALLBACK (gtk_main_quit), NULL);
   g_signal_connect (G_OBJECT (canvasPtr), "expose-event", G_CALLBACK (exposeCanvas), NULL);

   gtk_widget_show_all (windowPtr);

   gtk_main ();
   return (0);
}
#endif
