/* popUp multiPlane ground attack. V0.06.05 2008/01/13                      */
/* measConv.c measure unit conversion functions                             */
/* Copyright  2008 Author: Valerio Messina I-EFAA http://users.iol.it/efa  */
/* This file is part of PopUp.
   PopUp is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   PopUp is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.          */

#include "stdio.h"
#include "math.h"

#include "comType.h"
#include "coord.h"
#include "pupData.h"

/* Miles in metri */
errOk mile2meter (double* milePtr, double* meterPtr) {
   if (milePtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s milePtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (meterPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s meterPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (*milePtr<0.0) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s mile:%f must be positive\n", __FUNCTION__, *milePtr);
      return (ERROR);
   }
   *meterPtr = *milePtr * Mile;
   return (OK);
}

/* metri in Miles */
errOk meter2mile (double* meterPtr, double* milePtr) {
   if (meterPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s meterPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (milePtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s milePtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (*meterPtr<0.0) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s meter:%f must be positive\n", __FUNCTION__, *meterPtr);
      return (ERROR);
   }
   *milePtr = *meterPtr / Mile;
   return (OK);
}

/* feet in metri */
errOk feet2meter (double* feetPtr, double* meterPtr) {
   if (feetPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s feetPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (meterPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s meterPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (*feetPtr<0.0) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s feet:%f must be positive\n", __FUNCTION__, *feetPtr);
      return (ERROR);
   }

   *meterPtr = *feetPtr * Feet;
   return (OK);
}

/* metri in feet */
errOk meter2feet (double* meterPtr, double* feetPtr) {
   if (meterPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s meterPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (feetPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s feetPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (*meterPtr<0.0) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s meter:%f must be positive\n", __FUNCTION__, *meterPtr);
      return (ERROR);
   }
   *feetPtr = *meterPtr / Feet;
   return (OK);
}

/* arrotonda in modo corretto positivi e negativi ed  portabile */
/* -1.5=-2, -1.4=-1, -0.5=-1, -0.4=0, 0.4=0, 0.5=1, 1.4=1, 1.5=2 */
long int myint (double num) {
   if (num<0) num-=.6;
   else num+=.5;
   return ((long int)trunc (num));
}

/* tronca correttamente in unsigned long int */
unsigned long int mytru (double num) {
   long int trunc;
   trunc = myint (num - 0.5);
   if (trunc<0) trunc = 0;
   if (num - trunc > 0.9999999999996) {
      trunc++;
      if (pupDbg>=PRINTVERBOSE) printf ("INC\n");
   }
   return (trunc);
}

/* funzione opposta alla 'signbit', che restituisce true se l'argomento  <0
   restituisce -1.0 se l'argomento  true, altrimenti restituisce 1.0 */
double sgn2one (bool sgn) {
   if (sgn) return (-1.0);
   return (1.0);
}

/* converte gradi,primi,secondi interi in gradi decimali */
errOk degInt2deg (degTy* degPtr) {
   //bool sign;
   if (degPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s degPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (pupDbg>=PRINTALL) printf ("%s secInt:%u\n", __FUNCTION__, degPtr->secInt);
   if (pupDbg>=PRINTALL) printf ("%s priInt:%u\n", __FUNCTION__, degPtr->priInt);
   if (pupDbg>=PRINTALL) printf ("%s degInt:%u\n", __FUNCTION__, degPtr->degInt);
   if (pupDbg>=PRINTALL) printf ("%s sgnInt:%u\n", __FUNCTION__, degPtr->sgnInt);
   if (degPtr->secInt > 59) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s secInt:%u must be <60\n", __FUNCTION__, degPtr->secInt);
      return (ERROR);
   }
   if (degPtr->priInt > 59) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s priInt:%u must be <60\n", __FUNCTION__, degPtr->priInt);
      return (ERROR);
   }
   if (degPtr->degInt > 360) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s degInt:%u must be in range -180:+360\n", __FUNCTION__, degPtr->degInt);
      return (ERROR);
   }
   degPtr->sec = degPtr->secInt;
   degPtr->pri = degPtr->priInt + degPtr->secInt/60.0;
   //sign = signbit (degPtr->degInt);
   //if (sgnInt) {
   //   degPtr->deg = degPtr->degInt - degPtr->pri/60.0;
   //} else {
   //   degPtr->deg = degPtr->degInt + degPtr->pri/60.0;
   //}
   degPtr->deg = sgn2one (degPtr->sgnInt) * (degPtr->degInt + degPtr->pri/60.0);
   return (OK);
}

/* converte gradi decimali in gradi,primi,secondi interi */
errOk deg2degInt (degTy* degPtr) {
   if (degPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s degPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (degPtr->sec >= 60.0) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s sec:%f must be <60\n", __FUNCTION__, degPtr->sec);
      return (ERROR);
   }
   if (degPtr->pri >= 60.0) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s pri:%f must be <60\n", __FUNCTION__, degPtr->pri);
      return (ERROR);
   }
   if ((degPtr->deg < -180.0) || (degPtr->deg > 360.0)) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s deg:%f must be in range -180:+360\n", __FUNCTION__, degPtr->deg);
      return (ERROR);
   }
   degPtr->sgnInt = signbit (degPtr->deg);           // double    in boolean
   degPtr->degInt = mytru (fabs(degPtr->deg)); // double    in u16 verso 0
   degPtr->priInt = mytru (     degPtr->pri ); // double>=0 in u08 per difetto
   degPtr->secInt = myint (     degPtr->sec ); // double>=0 in u08 pi vicino
   return (OK);
}

/* ,  ',  ' " in rad */
errOk deg2rad (degTy* degPtr, double* radPtr) {
   if (degPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s degPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (radPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s radPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if ((degPtr->deg < -180.0) || (degPtr->deg > 360.0)) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s deg:%f must be in range -180:+360\n", __FUNCTION__, degPtr->deg);
      return (ERROR);
   }
   *radPtr = degPtr->deg * M_PI / 180.0;
   return (OK);
}

/* rad in ,  ',  ' " */
errOk rad2deg (double* radPtr, degTy* degPtr) {
   if (radPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s radPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if (degPtr==NULL) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s degPtr point to NULL\n", __FUNCTION__);
      return (ERROR);
   }
   if ((*radPtr<-M_PI)||(*radPtr>2.0*M_PI)) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad:%f out of range -PI:2PI\n", __FUNCTION__, *radPtr);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("\n_rad:%f\n", *radPtr);
   degPtr->deg = *radPtr * 180.0 / M_PI;
   if (pupDbg>=PRINTVERBOSE) printf ("_deg:%f\n", degPtr->deg);
   degPtr->sgnInt = signbit (degPtr->deg);           // double    in boolean
   if (pupDbg>=PRINTVERBOSE) printf ("_sgnInt:%f\n", sgn2one(degPtr->sgnInt));
   degPtr->degInt = mytru (fabs(degPtr->deg)); // double    in u16 verso 0
   if (pupDbg>=PRINTVERBOSE) printf ("_degInt:%u\n", degPtr->degInt);
   degPtr->pri = (fabs(degPtr->deg) - degPtr->degInt) * 60.0;
   if (pupDbg>=PRINTVERBOSE) printf ("_pri:%f\n", degPtr->pri);
   degPtr->priInt = mytru (     degPtr->pri ); // double>=0 in u08 per difetto
   if (pupDbg>=PRINTVERBOSE) printf ("_priInt:%u\n", degPtr->priInt);
   degPtr->sec = (     degPtr->pri  - degPtr->priInt) * 60.0;
   if (pupDbg>=PRINTVERBOSE) printf ("_sec:%f\n", degPtr->sec);
   degPtr->secInt = myint (     degPtr->sec ); // double>=0 in u08 pi vicino
   if (pupDbg>=PRINTVERBOSE) printf ("_secInt:%u\n", degPtr->secInt);
   return (OK);
}

#if (pupDbg>=PRINTF)
int try2 () {
   double rad;
   degTy deg;
   errOk ret;
   deg.sgnInt = 0;
   deg.degInt = 45;
   deg.priInt = 29;
   deg.secInt = 0;
   ret = degInt2deg (&deg);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s degInt2deg returned not OK\n", __FUNCTION__);
      return (ERROR);
   }
   ret = deg2rad (&deg, &rad);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned not OK\n", __FUNCTION__);
      return (ERROR);
   }
   printf ("deg:%9.5f pri:%6.3f sec:%6.3f ", deg.deg, deg.pri, deg.sec);
   printf ("degInt:%3u priInt:%2u secInt:%2u ", deg.degInt, deg.priInt, deg.secInt);
   printf ("rad:%.4f\n\n", rad);
   for (rad=0.0; rad<2*M_PI; rad+=1.0) {
      ret = rad2deg (&rad, &deg);
      if (ret!=OK) {
         if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned not OK\n", __FUNCTION__);
         return (ERROR);
      }
      printf ("rad:%1.0f ", rad);
      printf ("deg:%9.5f pri:%6.3f sec:%6.3f ", deg.deg, deg.pri, deg.sec);
      printf ("degInt:%3u priInt:%2u secInt:%2u\n", deg.degInt, deg.priInt, deg.secInt);
   }
   return (OK);
}

int try3 () {
   double deg;
   long int deg1;
   long int deg2;
   double deg3;
   long int deg4;
   for (deg=-3.0; deg<3.01; deg=deg+0.1) {
      deg1 = lrint  (deg);
      deg2 = lround (deg);
      deg3 = trunc (deg);
      deg4 = myint (deg);
      printf ("deg:%f lrint:%ld lround:%ld trunc:%f myint:%ld\n", deg, deg1, deg2, deg3, deg4);
   }
   return OK;
}
#endif
