/* popUp multiPlane ground attack. V0.06.05 2008/01/13                      */
/* pupCalc.c do calc on points needed to draw the graph                     */
/* Copyright  2008 Author: Valerio Messina I-EFAA http://users.iol.it/efa  */
/* This file is part of PopUp.
   PopUp is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   PopUp is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.          */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>                                                     // abs
#include <math.h>                                                   // signbit

#include "coord.h"
#include "pupData.h"

/* these are common variables storing values for points before calculations */
/* widget that belongs to:           Plane:     0   1   2   3 */
signed   short splLatDeg        ; // spinbutton13 196 216 236 
unsigned short splLatPri        ; // spinbutton14 197 217 237 
signed   short splLonDeg        ; // spinbutton16 198 218 238 
unsigned short splLonPri        ; // spinbutton17 199 219 239 
double         vrpFeet[Planes]  ; // spinbutton 6 180 200 220 
double         vrpAngl[Planes]  ; // spinbutton 5 181 201 221 
double         vipFeet[Planes]  ; // spinbutton 8 182 202 222 
double         vipAngl[Planes]  ; // spinbutton 7 183 203 223 
double         oa1Feet[Planes]  ; // spinbutton10 184 204 224 
double         oa1Angl[Planes]  ; // spinbutton 9 185 205 225 
signed   short tarLatDeg        ; // spinbutton18 186 206 226 
unsigned short tarLatPri        ; // spinbutton19 187 207 227 
signed   short tarLonDeg        ; // spinbutton20 188 208 228 
unsigned short tarLonPri        ; // spinbutton21 189 209 229 
double         oa2Feet[Planes]  ; // spinbutton 2 190 210 230 
double         oa2Angl[Planes]  ; // spinbutton 1 191 211 231 
signed   short rejLatDeg        ; // spinbutton22 192 212 232 
unsigned short rejLatPri        ; // spinbutton23 193 213 233 
signed   short rejLonDeg        ; // spinbutton24 194 214 234 
unsigned short rejLonPri        ; // spinbutton25 195 215 235 

/* this is common variable storing values for points after calculations */
pointTy points[Planes][Points]; /* 7 punti:   0    1   2   3     4    5    6
                                            Split,VRP,VIP,OA1,Target,OA2,Rejoin */

void pointsReset () { /* azzera il vettore di strutture */
   memset (points, 0, sizeof (points));
}

/* set defaults value for all points of a plane */
errOk pointsDefaultSet (u08 plane) {
   if (plane>=Planes) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s plane:%u must be <%u\n", __FUNCTION__, plane, Planes);
      return (ERROR);
   }
   /* plane 0 */
   splLatDeg        =    43  ; // was 45
   splLatPri        =     0  ; // was 19
   splLonDeg        =    11  ; // was  9
   splLonPri        =    20  ; // was 32
   vrpFeet[plane]   = 44031.0; // circa 13,42 Km
   vrpAngl[plane]   =   135.6;
   vipFeet[plane]   = 25802.0; // circa 7,864 Km
   vipAngl[plane]   =   135.6;
   oa1Feet[plane]   = 15288.0; // circa 4,659 Km
   oa1Angl[plane]   =   164.6;
   tarLatDeg        =    43  ; // was 45
   tarLatPri        =     0  ; // was 30
   tarLonDeg        =    11  ; // was  9
   tarLonPri        =     0  ; // was 30
   oa2Feet[plane]   =  2715.0; // circa 0,827 Km
   oa2Angl[plane]   =     0.0;
   rejLatDeg        =    43  ; // was 45
   rejLatPri        =    20  ; // was 41
   rejLonDeg        =    11  ; // was  9
   rejLonPri        =     0  ; // was 41
   switch (plane) {
   case 0:
      break;
   case 1:
      vrpAngl[plane]   = 225.6;
      vipAngl[plane]   = 225.6;
      oa1Angl[plane]   = 254.6;
      oa2Angl[plane]   =  90.0;
      break;
   case 2:
      vrpAngl[plane]   = 315.6;
      vipAngl[plane]   = 315.6;
      oa1Angl[plane]   = 344.6;
      oa2Angl[plane]   = 180.0;
      break;
   case 3:
      vrpAngl[plane]   =  45.6;
      vipAngl[plane]   =  45.6;
      oa1Angl[plane]   =  74.6;
      oa2Angl[plane]   = 270.0;
      break;
   default:
      if (pupDbg>=PRINTERROR) printf ("ERROR %s plane:%u should be <%u\n", __FUNCTION__, plane, Planes);
      return (ERROR);
   }
   return (OK);
}

/* Dati        : Coordinate polari punti: VRP, VIP, OA1, OA2
                 Coordinate cartesiane punti: Split, Target, Rejoin
   Incognite   : Coordinate cartesiane punti: VRP, VIP, OA1, OA2
                 Coordinate polari punti: Split, Target, Rejoin
   Procedimento: Sapendo le coordinate cartesiane di Target,
                 calcolo le coordinate cartesiane di VRP, VIP, OA1/2
                 poi calcolo le coordinate polari di Split, Target, Rejoin
*/

/* esegui i calcoli su tutti i punti di un aereo (parametrico) */
errOk calcCoords (u08 plane) {
   degTy deg, deg1, deg2, deg3;
   errOk ret;
   if (plane>=Planes) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s plane:%u must be <%u\n", __FUNCTION__, plane, Planes);
      return (ERROR);
   }
   /* ----------------------------- TARGET -------------------------------- */
   points[plane][Target].rel.ran = 0; // Plane 0, il Target  al centro
   points[plane][Target].rel.bea = 0; // Plane 0, il Target  al centro
   deg.sgnInt = signbit (tarLatDeg);
   deg.degInt = abs (tarLatDeg);
   deg.priInt = tarLatPri;
   deg.secInt = 0;
   ret = degInt2deg (&deg);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s degInt2deg returned not OK\n", __FUNCTION__);
      return (ERROR);
   }
   ret = deg2rad (&deg, &points[plane][Target].abs.latRad); // Plane 0, Target
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s deg2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   deg.sgnInt = signbit (tarLonDeg);
   deg.degInt = abs (tarLonDeg);
   deg.priInt = tarLonPri;
   deg.secInt = 0;
   ret = degInt2deg (&deg);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s degInt2deg returned not OK\n", __FUNCTION__);
      return (ERROR);
   }
   ret = deg2rad (&deg, &points[plane][Target].abs.lonRad); // Plane 0, Target
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s deg2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2metr (&points[plane][Target].abs); // Plane 0, Split
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2metr returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = cart2pol (&points[plane][Target].abs); // superfluo
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s cart2pol returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   /* ------------------------------ SPLIT -------------------------------- */
   deg.sgnInt = signbit(splLatDeg);
   deg.degInt = abs (splLatDeg);
   deg.priInt = splLatPri;
   deg.secInt = 0;
   ret = degInt2deg (&deg);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s degInt2deg returned not OK\n", __FUNCTION__);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("latDeg:%f\n", deg.deg);
   ret = deg2rad (&deg, &points[plane][Split].abs.latRad); // Plane 0, Split
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s deg2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("lat:%f\n", points[plane][Split].abs.latRad);
   deg.sgnInt = signbit (splLonDeg);
   deg.degInt = abs (splLonDeg);
   deg.priInt = splLonPri;
   deg.secInt = 0;
   ret = degInt2deg (&deg);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s degInt2deg returned not OK\n", __FUNCTION__);
      return (ERROR);
   }
   ret = deg2rad (&deg, &points[plane][Split].abs.lonRad); // Plane 0, Split
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s deg2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("lon:%f\n", points[plane][Split].abs.lonRad);
   ret = rad2metr (&points[plane][Split].abs); // Plane 0, Split
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2metr returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("lat:%f lon:%f\n", points[plane][Split].abs.lat, points[plane][Split].abs.lon);
   ret = cart2pol (&points[plane][Split].abs); // superfluo
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s cart2pol returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   points[plane][Split].rel.lat = points[plane][Split].abs.lat - points[plane][Target].abs.lat;
   points[plane][Split].rel.lon = points[plane][Split].abs.lon - points[plane][Target].abs.lon;
   /*#include "math.h"
   points[plane][Split].rel.lat = 10 *2.0*M_PI/360.0 * EarthRadius;
   points[plane][Split].rel.lon =  1 *2.0*M_PI/360.0 * EarthRadius;*/
   if (pupDbg>=PRINTVERBOSE) printf ("lat:%f\n", points[plane][Split].rel.lat);
   if (pupDbg>=PRINTVERBOSE) printf ("lon:%f\n", points[plane][Split].rel.lon);
   ret = metr2rad (&points[plane][Split].rel); // superfluo
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s metr2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = cart2pol (&points[plane][Split].rel); // Plane 0, Split
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s cart2pol returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("ran:%f\n", points[plane][Split].rel.ran);
   if (pupDbg>=PRINTVERBOSE) printf ("bea:%f\n", points[plane][Split].rel.bea);
   /*points[plane][Split].rel.bea=3;*/
   /* ------------------------------- VRP --------------------------------- */
   ret = feet2meter (&vrpFeet[plane], &points[plane][VRP].rel.ran); // Plane 0, VRP
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s feet2meter returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   deg.deg = vrpAngl[plane];
   ret = deg2rad (&deg, &points[plane][VRP].rel.bea); // Plane 0, VRP
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s deg2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   /*#include "math.h"
   points[plane][VRP].rel.bea = 89 *2.0*M_PI/360.0;*/
   if (pupDbg>=PRINTVERBOSE) printf ("bea:%f\n", points[plane][VRP].rel.bea);
   ret = pol2cart (&points[plane][VRP].rel); // Plane 0, VRP
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s pol2cart returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("lat:%f lon:%f\n", points[plane][VRP].rel.lat, points[plane][VRP].rel.lon);
   ret = metr2rad (&points[plane][VRP].rel); // superfluo
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s metr2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("lat:%f lon:%f\n", points[plane][VRP].rel.latRad, points[plane][VRP].rel.lonRad);
   points[plane][VRP].abs.lat = points[plane][Target].abs.lat + points[plane][VRP].rel.lat;
   points[plane][VRP].abs.lon = points[plane][Target].abs.lon + points[plane][VRP].rel.lon;
   ret = cart2pol (&points[plane][VRP].abs); // Plane 0, VRP
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s cart2pol returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = metr2rad (&points[plane][VRP].abs); // superfluo
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s metr2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   /* ------------------------------- VIP --------------------------------- */
   ret = feet2meter (&vipFeet[plane], &points[plane][VIP].rel.ran); // Plane 0, VIP
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s feet2meter returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   deg.deg = vipAngl[plane];
   ret = deg2rad (&deg, &points[plane][VIP].rel.bea); // Plane 0, VIP
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s deg2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = pol2cart (&points[plane][VIP].rel); // Plane 0, VIP
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s pol2cart returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = metr2rad (&points[plane][VIP].rel); // superfluo
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s metr2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   points[plane][VIP].abs.lat = points[plane][Target].abs.lat + points[plane][VIP].rel.lat;
   points[plane][VIP].abs.lon = points[plane][Target].abs.lon + points[plane][VIP].rel.lon;
   ret = cart2pol (&points[plane][VIP].abs); // Plane 0, VIP
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s cart2pol returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = metr2rad (&points[plane][VIP].abs); // superfluo
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s metr2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   /* ------------------------------- OA1 --------------------------------- */
   ret = feet2meter (&oa1Feet[plane], &points[plane][OA1].rel.ran); // Plane 0, OA1
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s feet2meter returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   deg.deg = oa1Angl[plane];
   ret = deg2rad (&deg, &points[plane][OA1].rel.bea); // Plane 0, OA1
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s deg2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = pol2cart (&points[plane][OA1].rel); // Plane 0, OA1
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s pol2cart returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = metr2rad (&points[plane][OA1].rel); // superfluo
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s metr2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   points[plane][OA1].abs.lat = points[plane][Target].abs.lat + points[plane][OA1].rel.lat;
   points[plane][OA1].abs.lon = points[plane][Target].abs.lon + points[plane][OA1].rel.lon;
   ret = cart2pol (&points[plane][OA1].abs); // Plane 0, OA1
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s cart2pol returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = metr2rad (&points[plane][OA1].abs); // superfluo
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s metr2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   /* ------------------------------- OA2 --------------------------------- */
   ret = feet2meter (&oa2Feet[plane], &points[plane][OA2].rel.ran); // Plane 0, OA2
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s feet2meter returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   deg.deg = oa2Angl[plane];
   ret = deg2rad (&deg, &points[plane][OA2].rel.bea); // Plane 0, OA2
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s deg2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = pol2cart (&points[plane][OA2].rel); // Plane 0, OA2
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s pol2cart returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = metr2rad (&points[plane][OA2].rel); // superfluo
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s metr2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   points[plane][OA2].abs.lat = points[plane][Target].abs.lat + points[plane][OA2].rel.lat;
   points[plane][OA2].abs.lon = points[plane][Target].abs.lon + points[plane][OA2].rel.lon;
   ret = cart2pol (&points[plane][OA2].abs); // Plane 0, OA2
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s cart2pol returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = metr2rad (&points[plane][OA2].abs); // superfluo
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s metr2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   /* ----------------------------- REJOIN -------------------------------- */
   deg.sgnInt = signbit (rejLatDeg);
   deg.degInt = abs (rejLatDeg);
   deg.priInt = rejLatPri;
   deg.secInt = 0;
   ret = degInt2deg (&deg);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s degInt2deg returned not OK\n", __FUNCTION__);
      return (ERROR);
   }
   ret = deg2rad (&deg, &points[plane][Rejoin].abs.latRad); // Plane 0, Rejoin
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s deg2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   deg.sgnInt = signbit (rejLonDeg);
   deg.degInt = abs (rejLonDeg);
   deg.priInt = rejLonPri;
   deg.secInt = 0;
   ret = degInt2deg (&deg);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s degInt2deg returned not OK\n", __FUNCTION__);
      return (ERROR);
   }
   ret = deg2rad (&deg, &points[plane][Rejoin].abs.lonRad); // Plane 0, Rejoin
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s deg2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2metr (&points[plane][Rejoin].abs); // Plane 0, Rejoin
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2metr returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   points[plane][Rejoin].rel.lat = points[plane][Rejoin].abs.lat - points[plane][Target].abs.lat;
   points[plane][Rejoin].rel.lon = points[plane][Rejoin].abs.lon - points[plane][Target].abs.lon;
   ret = metr2rad (&points[plane][Rejoin].rel); // superfluo
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s metr2rad returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = cart2pol (&points[plane][Rejoin].rel); // Plane 0, Rejoin
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s cart2pol returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = cart2pol (&points[plane][Rejoin].abs); // superfluo
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s cart2pol returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }

   /* ----------------------------- OUTPUT -------------------------------- */
   if (pupDbg>=PRINTVERBOSE) printf ("Point_ __Absolute_to_Equator/Greenwich_  _______Relative_to_Target_______\n");
   if (pupDbg>=PRINTVERBOSE) printf ("Name__ Latitud Longitud Range_m_ Bearin  Latitud Longitud Range_m_ Bearin\n");
   if (pupDbg>=PRINTDEBUG) printf ("Point_ __Absolute_to_Equator/Greenwich_  __________Relative_to_Target__________\n");
   if (pupDbg>=PRINTDEBUG) printf ("Name__ Latitud Longitud Range_m_ Bearin  Latitude__ Longitude__ Range_m_ Bearin\n");
   /* ------------------------------ SPLIT -------------------------------- */
   ret = rad2deg (&points[plane][Split].abs.latRad, &deg1);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   /*printf ("Split  %f\n", deg1.deg);*/
   ret = rad2deg (&points[plane][Split].abs.lonRad, &deg2);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][Split].abs.bea, &deg3);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("Split  %c%2u%2u' %c%3u%2u' %8.0f %5.1f  ", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][Split].abs.ran, deg3.deg);
   if (pupDbg>=PRINTDEBUG) printf ("Split  %c%2u%2u' %c%3u%2u' %8.0f %5.1f  ", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][Split].abs.ran, deg3.deg);
   ret = rad2deg (&points[plane][Split].rel.latRad, &deg1);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][Split].rel.lonRad, &deg2);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][Split].rel.bea, &deg3);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("%c%2u%2u' %c%3u%2u' %8.0f %5.1f\n", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][Split].rel.ran, deg3.deg);
   if (pupDbg>=PRINTDEBUG) printf ("%c%2u%2u'%2u\" %c%3u%2u'%2u\" %8.0f %5.1f\n", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg1.secInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, deg2.secInt, points[plane][Split].rel.ran, deg3.deg);
   /* ------------------------------- VRP --------------------------------- */
   ret = rad2deg (&points[plane][VRP].abs.latRad, &deg1);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][VRP].abs.lonRad, &deg2);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][VRP].abs.bea, &deg3);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("VRP    %c%2u%2u' %c%3u%2u' %8.0f %5.1f  ", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][VRP].abs.ran, deg3.deg);
   if (pupDbg>=PRINTDEBUG) printf ("VRP    %c%2u%2u' %c%3u%2u' %8.0f %5.1f  ", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][VRP].abs.ran, deg3.deg);
   if (pupDbg>=PRINTALL) printf ("latRad:%f\n", points[plane][VRP].rel.latRad);
   ret = rad2deg (&points[plane][VRP].rel.latRad, &deg1);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTALL) printf ("deg:%f\n", deg1.deg);
   if (pupDbg>=PRINTALL) printf ("lonRad:%f\n", points[plane][VRP].rel.lonRad);
   /*points[plane][VRP].rel.lonRad = 0;*/
   ret = rad2deg (&points[plane][VRP].rel.lonRad, &deg2);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTALL) printf ("deg:%f\n", deg2.deg);
   ret = rad2deg (&points[plane][VRP].rel.bea, &deg3);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("%c%2u%2u' %c%3u%2u' %8.0f %5.1f\n", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][VRP].rel.ran, deg3.deg);
   if (pupDbg>=PRINTDEBUG) printf ("%c%2u%2u'%2u\" %c%3u%2u'%2u\" %8.0f %5.1f\n", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg1.secInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, deg2.secInt, points[plane][VRP].rel.ran, deg3.deg);
   /* ------------------------------- VIP --------------------------------- */
   ret = rad2deg (&points[plane][VIP].abs.latRad, &deg1);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][VIP].abs.lonRad, &deg2);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][VIP].abs.bea, &deg3);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("VIP    %c%2u%2u' %c%3u%2u' %8.0f %5.1f  ", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][VIP].abs.ran, deg3.deg);
   if (pupDbg>=PRINTDEBUG) printf ("VIP    %c%2u%2u' %c%3u%2u' %8.0f %5.1f  ", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][VIP].abs.ran, deg3.deg);
   ret = rad2deg (&points[plane][VIP].rel.latRad, &deg1);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][VIP].rel.lonRad, &deg2);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][VIP].rel.bea, &deg3);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("%c%2u%2u' %c%3u%2u' %8.0f %5.1f\n", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][VIP].rel.ran, deg3.deg);
   if (pupDbg>=PRINTDEBUG) printf ("%c%2u%2u'%2u\" %c%3u%2u'%2u\" %8.0f %5.1f\n", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg1.secInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, deg2.secInt, points[plane][VIP].rel.ran, deg3.deg);
   /* ------------------------------- OA1 --------------------------------- */
   ret = rad2deg (&points[plane][OA1].abs.latRad, &deg1);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][OA1].abs.lonRad, &deg2);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][OA1].abs.bea, &deg3);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("OA1    %c%2u%2u' %c%3u%2u' %8.0f %5.1f  ", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][OA1].abs.ran, deg3.deg);
   if (pupDbg>=PRINTDEBUG) printf ("OA1    %c%2u%2u' %c%3u%2u' %8.0f %5.1f  ", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][OA1].abs.ran, deg3.deg);
   ret = rad2deg (&points[plane][OA1].rel.latRad, &deg1);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][OA1].rel.lonRad, &deg2);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][OA1].rel.bea, &deg3);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("%c%2u%2u' %c%3u%2u' %8.0f %5.1f\n", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][OA1].rel.ran, deg3.deg);
   if (pupDbg>=PRINTDEBUG) printf ("%c%2u%2u'%2u\" %c%3u%2u'%2u\" %8.0f %5.1f\n", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg1.secInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, deg2.secInt, points[plane][OA1].rel.ran, deg3.deg);
   /* ----------------------------- TARGET -------------------------------- */
   ret = rad2deg (&points[plane][Target].abs.latRad, &deg1);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][Target].abs.lonRad, &deg2);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][Target].abs.bea, &deg3);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("Target %c%2u%2u' %c%3u%2u' %8.0f %5.1f  ", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][Target].abs.ran, deg3.deg);
   if (pupDbg>=PRINTDEBUG) printf ("Target %c%2u%2u' %c%3u%2u' %8.0f %5.1f  ", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][Target].abs.ran, deg3.deg);
   ret = rad2deg (&points[plane][Target].rel.latRad, &deg1);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][Target].rel.lonRad, &deg2);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][Target].rel.bea, &deg3);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("%c%2u%2u' %c%3u%2u' %8.0f %5.1f\n", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][Target].rel.ran, deg3.deg);
   if (pupDbg>=PRINTDEBUG) printf ("%c%2u%2u'%2u\" %c%3u%2u'%2u\" %8.0f %5.1f\n", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg1.secInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, deg2.secInt, points[plane][Target].rel.ran, deg3.deg);
   /* ------------------------------- OA2 --------------------------------- */
   ret = rad2deg (&points[plane][OA2].abs.latRad, &deg1);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][OA2].abs.lonRad, &deg2);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][OA2].abs.bea, &deg3);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("OA2    %c%2u%2u' %c%3u%2u' %8.0f %5.1f  ", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][OA2].abs.ran, deg3.deg);
   if (pupDbg>=PRINTDEBUG) printf ("OA2    %c%2u%2u' %c%3u%2u' %8.0f %5.1f  ", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][OA2].abs.ran, deg3.deg);
   ret = rad2deg (&points[plane][OA2].rel.latRad, &deg1);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][OA2].rel.lonRad, &deg2);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][OA2].rel.bea, &deg3);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("%c%2u%2u' %c%3u%2u' %8.0f %5.1f\n", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][OA2].rel.ran, deg3.deg);
   if (pupDbg>=PRINTDEBUG) printf ("%c%2u%2u'%2u\" %c%3u%2u'%2u\" %8.0f %5.1f\n", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg1.secInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, deg2.secInt, points[plane][OA2].rel.ran, deg3.deg);
   /* ----------------------------- REJOIN -------------------------------- */
   ret = rad2deg (&points[plane][Rejoin].abs.latRad, &deg1);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][Rejoin].abs.lonRad, &deg2);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][Rejoin].abs.bea, &deg3);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("Rejoin %c%2u%2u' %c%3u%2u' %8.0f %5.1f  ", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][Rejoin].abs.ran, deg3.deg);
   if (pupDbg>=PRINTDEBUG) printf ("Rejoin %c%2u%2u' %c%3u%2u' %8.0f %5.1f  ", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][Rejoin].abs.ran, deg3.deg);
   ret = rad2deg (&points[plane][Rejoin].rel.latRad, &deg1);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][Rejoin].rel.lonRad, &deg2);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   ret = rad2deg (&points[plane][Rejoin].rel.bea, &deg3);
   if (ret!=OK) {
      if (pupDbg>=PRINTERROR) printf ("ERROR %s rad2deg returned:%d not OK\n", __FUNCTION__, ret);
      return (ERROR);
   }
   if (pupDbg>=PRINTVERBOSE) printf ("%c%2u%2u' %c%3u%2u' %8.0f %5.1f\n", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, points[plane][Rejoin].rel.ran, deg3.deg);
   if (pupDbg>=PRINTDEBUG) printf ("%c%2u%2u'%2u\" %c%3u%2u'%2u\" %8.0f %5.1f\n", deg1.sgnInt?'-':' ', deg1.degInt, deg1.priInt, deg1.secInt, deg2.sgnInt?'-':' ', deg2.degInt, deg2.priInt, deg2.secInt, points[plane][Rejoin].rel.ran, deg3.deg);

   return (OK);
}

