/* popUp multiPlane ground attack. V0.06.05 2008/01/13                      */
/* Copyright  2008 Author: Valerio Messina I-EFAA http://users.iol.it/efa    */
/* pupData.h interface to reference data specific to popUp                  */
/* This file is part of PopUp.
   PopUp is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   PopUp is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.          */

#ifndef _INCpupDatah
#define _INCpupDatah

#include "comType.h"                         // inutile incluso gi da coord.h
#include "coord.h"

#define AppletName  "PopUp Multiplane Ground Attack"
#define AppletVer   "Ver.0.06.05 2008/01/13"
#define AppletAuth  "Valerio Messina I-EFAA http://users.iol.it/efa"

#define Mile        1852.0              /* lunghezza miglio marino in metri */
#define Feet        0.3048                /* lunghezza di un piede in metri */
#define EarthRadius 6371000.0            /* raggio medio terrestre in metri */

#define Planes      4

#define Split       0
#define VRP         1
#define VIP         2
#define OA1         3
#define Target      4
#define OA2         5
#define Rejoin      6
#define Points      7                                       // numero di punti

typedef struct point { /* all measures are SI */
   coordTy abs; // coordinate assolute riferite all'Equatore+Greenwich
   coordTy rel; // coordinate relative riferite al Bulls/Target
} pointTy;

extern unsigned short int pupDbg; /* Interaction level */

/* these are common variables storing values for points before calculations */
extern signed   short splLatDeg        ; // spinbutton13
extern unsigned short splLatPri        ; // spinbutton14
extern signed   short splLonDeg        ; // spinbutton16
extern unsigned short splLonPri        ; // spinbutton17
extern double         vrpFeet[Planes]  ; // spinbutton 6
extern double         vrpAngl[Planes]  ; // spinbutton 5
extern double         vipFeet[Planes]  ; // spinbutton 8
extern double         vipAngl[Planes]  ; // spinbutton 7
extern double         oa1Feet[Planes]  ; // spinbutton10
extern double         oa1Angl[Planes]  ; // spinbutton 9
extern signed   short tarLatDeg        ; // spinbutton18
extern unsigned short tarLatPri        ; // spinbutton19
extern signed   short tarLonDeg        ; // spinbutton20
extern unsigned short tarLonPri        ; // spinbutton21
extern double         oa2Feet[Planes]  ; // spinbutton 2
extern double         oa2Angl[Planes]  ; // spinbutton 1
extern signed   short rejLatDeg        ; // spinbutton22
extern unsigned short rejLatPri        ; // spinbutton23
extern signed   short rejLonDeg        ; // spinbutton24
extern unsigned short rejLonPri        ; // spinbutton25

/* this is common variable storing values for points after calculations */
extern pointTy points[Planes][Points]; /* 7 punti:   0    1   2   3     4    5    6
                                                   Split,VRP,VIP,OA1,Target,OA2,Rejoin */
/* Miles in metri */
errOk mile2meter (double* milePtr, double* meterPtr);

/* metri in Miles */
errOk meter2mile (double* meterPtr, double* milePtr);

/* feet in metri */
errOk feet2meter (double* feetPtr, double* meterPtr);

/* metri in feet */
errOk meter2feet (double* meterPtr, double* feetPtr);

/* converte gradi,primi,secondi interi in gradi decimali */
errOk degInt2deg (degTy* degPtr);

/* converte gradi decimali in gradi,primi,secondi interi */
errOk deg2degInt (degTy* degPtr);

/* ,  ',  ' " in rad */
errOk deg2rad (degTy* degPtr, double* radPtr);

/* rad in ,  ',  ' " */
errOk rad2deg (double* radPtr, degTy* degPtr);

/* azzera il vettore di strutture */
void pointsReset (void);

/* set defaults for all points of a plane */
errOk pointsDefaultSet (u08 plane);

/* imposta tutto al default. All'avvio e con Menu File, Load Default */
errOk allDefaultSet (void);

/* esegui i calcoli su tutti i punti di un aereo (parametrico) */
errOk calcCoords (u08 plane);

#endif /* _INCpupDatah */
